/*******************************************************************************
***  This file is part of Geometry.                                          ***
***                                                                          ***
***  Copyright (C) 2024                                                      ***
***  John Schneiderman <Licencing _AT_ Schneiderman _DOT_ me>                ***
***                                                                          ***
***  This program is free software: you can redistribute it and/or modify it ***
***  under the terms of the GNU Affero General Public License as published   ***
***  by the Free Software Foundation, either version 3 of the License, or    ***
***  (at your option) any later version.                                     ***
***                                                                          ***
***  This program is distributed in the hope that it will be useful, but     ***
***  WITHOUT ANY WARRANTY; without even the implied warranty of              ***
***  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                    ***
***  See the GNU Lesser General Public License for more details.             ***
***                                                                          ***
***  You should have received a copy of the GNU Lesser General Public License***
***  along with this program. If not, see <http://www.gnu.org/licenses/>.    ***
*******************************************************************************/
#ifndef PROCEDURAL_SHAPE_H
#define PROCEDURAL_SHAPE_H

#include <stdint.h>

#include "Cartesian.h"


/// The constant used as a return value for when data is invalid for a shape.
extern const int INVALID_SHAPE_DATA;

/**
 * @brief The geometric shapes supported.
 */
enum ShapeTypes
{
	Unknown,
	Circle,
	Line,
	Rectangle,
	Square,
	Triangle,
};
struct Circle
{
	/// The centre point of the circle's shape.
	struct Point centre_;
	/// The length of the circle's radius.
	int32_t radius_;
};
struct Line
{
	/// The starting point of the line segment.
	struct Point start_;
	/// The ending point of the line segment.
	struct Point end_;
};
struct Rectangle
{
	/// The left upper corner edge of the shape.
	struct Point leftEdge_;
	/// The horizontal distance of the shape.
	int32_t length_;
	/// The vertical distance of the shape.
	int32_t width_;
};
struct Square
{
	/// The left upper corner edge of the shape.
	struct Point leftEdge_;
	/// The horizontal distance of the shape.
	int32_t length_;
};
struct Triangle
{
	/// The first vertex point of the shape.
	struct Point vertex0_;
	/// The second vertex point of the shape.
	struct Point vertex1_;
	/// The third vertex point of the shape.
	struct Point vertex2_;
};

/**
 * @brief Converts a weak type safe value into the co-responding strong type value.
 *
 * @param weak The weak type value to convert.
 *
 * @return The strong type safe value for the supplied value when it is known, else Unknown.
 */
enum ShapeTypes toShapeTypes(const int weak);
/**
 * @brief Writes to the standard output buffer all the supported geometric shapes.
 *
 * @post The standard output buffer is filled with the supported shapes in the form:
 * ShapeIntegerValue - ShapeName.
 *
 * @return When successfully writes gives zero, a negative value when there is an error.
 */
int printfAllShapes(void);
/**
 * @brief Allocates dynamic memory of the requested shape.
 *
 * @param shape The geometric type to allocate.
 *
 * @return The allocated memory on success that must be released using free, or null on error.
 */
void* mallocShape(enum ShapeTypes shape);

//{ Circle

/**
 * @brief Reads in the values for a circle shape. The expected input form:
 * centre-abscissa centre-ordinate radius
 *
 * @param shape The shape object to read into.
 *
 * @post The standard input buffer is advanced from reading the values.
 *
 * @return The number of values successfully read or zero if there is a failure. Additionally, EOF
 * may be returned in the event of failure.
 */
int scanfCircle(struct Circle* shape);
/**
 * @brief Writes out the values of a circle shape.
 *
 * @param shape The shape object to write out to the standard output buffer.
 *
 * @post The standard output buffer is filled with the shape's values in the form:
 * (centre-abscissa, centre-ordinate)r-Radius.
 *
 * @return The number of characters written to the standard output buffer, on error a negative
 * value.
 */
int printfCircle(const struct Circle shape);

//}

//{ Line

/**
 * @brief Reads in the values for a line shape. The expected input form:
 * start-abscissa start-ordinate end-abscissa end-ordinate
 *
 * @param shape The shape object to read into.
 *
 * @post The standard input buffer is advanced from reading the values.
 *
 * @return The number of values successfully read or zero if there is a failure. Additionally, EOF
 * may be returned in the event of failure.
 */
int scanfLine(struct Line* shape);
/**
 * @brief Writes out the values of a line shape.
 *
 * @param shape The shape object to write out to the standard output buffer.
 *
 * @post The standard output buffer is filled with the shape's values in the form:
 * (start-abscissa start-ordinate)(end-abscissa end-ordinate).
 *
 * @return The number of characters written to the standard output buffer, on error a negative
 * value.
 */
int printfLine(const struct Line shape);

//}

//{ Rectangle

/**
 * @brief Reads in the values for a rectangle shape. The expected input form:
 * left-upper-abscissa left-upper-ordinate length width
 *
 * @param shape The shape object to read into.
 *
 * @post The standard input buffer is advanced from reading the values.
 *
 * @return The number of values successfully read or zero if there is a failure. Additionally, EOF
 * may be returned in the event of failure.
 */
int scanfRectangle(struct Rectangle* shape);
/**
 * @brief Writes out the values of a rectangle shape.
 *
 * @param shape The shape object to write out to the standard output buffer.
 *
 * @post The standard output buffer is filled with the shape's values in the form:
 * (left-upper-abscissa left-upper-ordinate)l-Length-w-Width.
 *
 * @return The number of characters written to the standard output buffer, on error a negative
 * value.
 */
int printfRectangle(const struct Rectangle shape);

//}

//{ Square

/**
 * @brief Reads in the values for a square shape. The expected input form:
 * left-upper-abscissa left-upper-ordinate length
 *
 * @param shape The shape object to read into.
 *
 * @post The standard input buffer is advanced from reading the values.
 *
 * @return The number of values successfully read or zero if there is a failure. Additionally, EOF
 * may be returned in the event of failure.
 */
int scanfSquare(struct Square* shape);
/**
 * @brief Writes out the values of a square shape.
 *
 * @param shape The shape object to write out to the standard output buffer.
 *
 * @post The standard output buffer is filled with the shape's values in the form:
 * (left-upper-abscissa left-upper-ordinate)l-Length.
 *
 * @return The number of characters written to the standard output buffer, on error a negative
 * value.
 */
int printfSquare(const struct Square shape);

//}

//{ Triangle

/**
 * @brief Reads in the values for a triangle shape. The expected input form:
 * first-abscissa first-ordinate second-abscissa second-ordinate third-abscissa third-ordinate
 *
 * @param shape The shape object to read into.
 *
 * @post The standard input buffer is advanced from reading the values.
 *
 * @return The number of values successfully read or zero if there is a failure. Additionally, EOF
 * may be returned in the event of failure.
 */
int scanfTriangle(struct Triangle* shape);
/**
 * @brief Writes out the values of a triangle shape.
 *
 * @param shape The shape object to write out to the standard output buffer.
 *
 * @post The standard output buffer is filled with the shape's values in the form:
 * (first-abscissa first-ordinate)(second-abscissa second-ordinate)(third-abscissa third-ordinate).
 *
 * @return The number of characters written to the standard output buffer, on error a negative
 * value.
 */
int printfTriangle(const struct Triangle shape);

//}

#endif
